/*
 *  othersyscalls.c
 *
 * Fractal Guru <reverse@put.as> http://reverse.put.as
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "structures.h"

extern open_func_t *real_open;
extern chmod_func_t *real_chmod;

// Hijack the open syscall
int our_open (struct proc *p, struct open_args *uap, int *retval)
{
	int error;
	size_t dummy = 0;
	char savedpath[MAXPATHLEN];
	char processname[MAXCOMLEN+1];
	
	proc_name(p->p_pid, processname, sizeof(processname));
	// copy from user space to kernel space
	//    copyinstr()   Copies a NUL-terminated string, at most len bytes long, from user-space address uaddr to
	//    kernel-space address kaddr.  The number of bytes actually copied, including the terminating 
	//	  NUL, is returned in *done.
	//	  int copyinstr(const void *uaddr, void *kaddr, size_t len, size_t *done);
	error = copyinstr(uap->path, (void *)savedpath, MAXPATHLEN, &dummy);
	if (!error)
		printf("[onyx-the-black-cat] open(%s, %x, %x) from PID %d (%s)\n", savedpath, uap->flags, uap->mode, p->p_pid, processname);
	else
		printf("[onyx-the-black-cat] open(?, %x, %x) from PID %d (%s)\n", uap->flags, uap->mode, p->p_pid, processname);
	// pass control to the real open function
	return real_open(p, uap, retval);
}


/*
 The system call function implements the system call. Its function prototype is 
 defined in the <sys/sysent.h> header as: 
 typedef int32_t sy_call_t(struct proc *, void *, int *);
 where struct proc * points to the currently calling process, and void * 
 points to the system call’s arguments’ structure, if there is any, and
 int * to the return value. 
 */
// bsd/vfs/vfs_syscalls.c
/* find the function that implements the syscal at bsd/kern/init_sysent.c */

// Hijack chmod syscall
int our_chmod (struct proc *p, struct chmod_args *uap, int *retval)
{
	int error;
	size_t dummy = 0;
	char savedpath[MAXPATHLEN];
	char processname[MAXCOMLEN+1];
	
	proc_name(p->p_pid, processname, sizeof(processname));
	//      chmod(const char *path, mode_t mode);
	error = copyinstr(uap->path, savedpath, MAXPATHLEN, &dummy);
	if (!error)
		printf("[onyx-the-black-cat] chmod requested to filename: %s from PID %d (%s)\n", savedpath, p->p_pid, processname);
	else
		printf("[onyx-the-black-cat] chmod requested to unknown filename from PID %d (%s)\n", p->p_pid, processname);
	// pass control to the real chmod function
	return real_chmod(p, uap, retval);
}

